@@ -1,4 +1,4 @@
-#if defined(GDK_PIXBUF_DISABLE_SINGLE_INCLUDES) && !defined (GDK_PIXBUF_H_INSIDE) && !defined (GDK_PIXBUF_COMPILATION)
+#if !defined (GDK_PIXBUF_H_INSIDE) && !defined (GDK_PIXBUF_COMPILATION)
 #error "Only <gdk-pixbuf/gdk-pixbuf.h> can be included directly."
 #endif
 
@@ -7,8 +7,8 @@
 
 #define GDK_PIXBUF_MAJOR (2)
 #define GDK_PIXBUF_MINOR (90)
-#define GDK_PIXBUF_MICRO (1)
-#define GDK_PIXBUF_VERSION "2.90.1"
+#define GDK_PIXBUF_MICRO (3)
+#define GDK_PIXBUF_VERSION "2.90.3"
 
 /* We prefix variable declarations so they can
  * properly get exported/imported from Windows DLLs.
