@@ -29,8 +29,6 @@
 #include <math.h>
 #include <string.h>
 
-#undef GDK_DISABLE_DEPRECATED /* We need gdk_drawable_get_size() */
-
 #include "gtkorientable.h"
 #include "gtkruler.h"
 #include "gtkprivate.h"
@@ -280,7 +278,6 @@ gtk_ruler_get_property (GObject      *object,
     }
 }
 
-#if 0
 /**
  * gtk_ruler_new:
  * @orientation: the ruler's orientation.
@@ -289,7 +286,7 @@ gtk_ruler_get_property (GObject      *object,
  *
  * Return value: a new #GtkRuler.
  *
- * Since: 2.16
+ * Since: 3.0
  **/
 GtkWidget *
 gtk_ruler_new (GtkOrientation orientation)
@@ -298,7 +295,6 @@ gtk_ruler_new (GtkOrientation orientation)
                        "orientation", orientation,
                        NULL);
 }
-#endif
 
 void
 gtk_ruler_set_metric (GtkRuler      *ruler,
@@ -481,6 +477,12 @@ gtk_ruler_unrealize (GtkWidget *widget)
       ruler->backing_store = NULL;
     }
 
+  if (ruler->non_gr_exp_gc)
+    {
+      g_object_unref (ruler->non_gr_exp_gc);
+      ruler->non_gr_exp_gc = NULL;
+    }
+
   GTK_WIDGET_CLASS (gtk_ruler_parent_class)->unrealize (widget);
 }
 
@@ -554,15 +556,15 @@ gtk_ruler_expose (GtkWidget      *widget,
   if (gtk_widget_is_drawable (widget))
     {
       GtkRuler *ruler = GTK_RULER (widget);
-      cairo_t *cr;
 
       gtk_ruler_draw_ticks (ruler);
       
-      cr = gdk_cairo_create (widget->window);
-      gdk_cairo_set_source_pixmap (cr, ruler->backing_store, 0, 0);
-      gdk_cairo_rectangle (cr, &event->area);
-      cairo_fill (cr);
-      cairo_destroy (cr);
+      gdk_draw_drawable (widget->window,
+			 ruler->non_gr_exp_gc,
+			 ruler->backing_store,
+			 0, 0, 0, 0,
+			 widget->allocation.width,
+			 widget->allocation.height);
       
       gtk_ruler_draw_pos (ruler);
     }
@@ -596,6 +598,12 @@ gtk_ruler_make_pixmap (GtkRuler *ruler)
 
   ruler->xsrc = 0;
   ruler->ysrc = 0;
+
+  if (!ruler->non_gr_exp_gc)
+    {
+      ruler->non_gr_exp_gc = gdk_gc_new (widget->window);
+      gdk_gc_set_exposures (ruler->non_gr_exp_gc, FALSE);
+    }
 }
 
 static void
@@ -854,15 +862,12 @@ gtk_ruler_real_draw_pos (GtkRuler *ruler)
 
 	  /*  If a backing store exists, restore the ruler  */
 	  if (ruler->backing_store)
-            {
-              cairo_t *cr = gdk_cairo_create (widget->window);
-
-              gdk_cairo_set_source_pixmap (cr, ruler->backing_store, 0, 0);
-              cairo_rectangle (cr, ruler->xsrc, ruler->ysrc, bs_width, bs_height);
-              cairo_fill (cr);
-
-              cairo_destroy (cr);
-            }
+	    gdk_draw_drawable (widget->window,
+			       widget->style->black_gc,
+			       ruler->backing_store,
+			       ruler->xsrc, ruler->ysrc,
+			       ruler->xsrc, ruler->ysrc,
+			       bs_width, bs_height);
 
           if (private->orientation == GTK_ORIENTATION_HORIZONTAL)
             {
