@@ -2,7 +2,6 @@
 /* Generated data (by glib-mkenums) */
 
 #undef GTK_DISABLE_DEPRECATED
-#define GTK_ENABLE_BROKEN
 #include "gtk.h"
 #include "gtkprivate.h"
 #include "gtkalias.h"
@@ -1458,21 +1457,6 @@ gtk_private_flags_get_type (void)
 }
 
 /* enumerations from "gtkprogressbar.h" */
-GType
-gtk_progress_bar_style_get_type (void)
-{
-    static GType etype = 0;
-    if (G_UNLIKELY(etype == 0)) {
-        static const GEnumValue values[] = {
-            { GTK_PROGRESS_CONTINUOUS, "GTK_PROGRESS_CONTINUOUS", "continuous" },
-            { GTK_PROGRESS_DISCRETE, "GTK_PROGRESS_DISCRETE", "discrete" },
-            { 0, NULL, NULL }
-        };
-        etype = g_enum_register_static (g_intern_static_string ("GtkProgressBarStyle"), values);
-    }
-    return etype;
-}
-
 GType
 gtk_progress_bar_orientation_get_type (void)
 {
@@ -1872,7 +1856,6 @@ gtk_widget_flags_get_type (void)
             { GTK_HAS_GRAB, "GTK_HAS_GRAB", "has-grab" },
             { GTK_RC_STYLE, "GTK_RC_STYLE", "rc-style" },
             { GTK_COMPOSITE_CHILD, "GTK_COMPOSITE_CHILD", "composite-child" },
-            { GTK_NO_REPARENT, "GTK_NO_REPARENT", "no-reparent" },
             { GTK_APP_PAINTABLE, "GTK_APP_PAINTABLE", "app-paintable" },
             { GTK_RECEIVES_DEFAULT, "GTK_RECEIVES_DEFAULT", "receives-default" },
             { GTK_DOUBLE_BUFFERED, "GTK_DOUBLE_BUFFERED", "double-buffered" },
