@@ -215,8 +215,6 @@ GType gtk_private_flags_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PRIVATE_FLAGS (gtk_private_flags_get_type ())
 
 /* enumerations from "gtkprogressbar.h" */
-GType gtk_progress_bar_style_get_type (void) G_GNUC_CONST;
-#define GTK_TYPE_PROGRESS_BAR_STYLE (gtk_progress_bar_style_get_type ())
 GType gtk_progress_bar_orientation_get_type (void) G_GNUC_CONST;
 #define GTK_TYPE_PROGRESS_BAR_ORIENTATION (gtk_progress_bar_orientation_get_type ())
 
