@@ -605,6 +605,24 @@ gtk_viewport_get_bin_window (GtkViewport *viewport)
   return viewport->bin_window;
 }
 
+/**
+ * gtk_viewport_get_view_window:
+ * @viewport: a #GtkViewport
+ *
+ * Gets the view window of the #GtkViewport.
+ *
+ * Return value: (transfer none): a #GdkWindow
+ *
+ * Since: 2.22
+ **/
+GdkWindow*
+gtk_viewport_get_view_window (GtkViewport *viewport)
+{
+  g_return_val_if_fail (GTK_IS_VIEWPORT (viewport), NULL);
+
+  return viewport->view_window;
+}
+
 static void
 gtk_viewport_realize (GtkWidget *widget)
 {
