@@ -20,6 +20,7 @@
 #include "config.h"
 
 #include <string.h>
+
 #include <gtk/gtk.h>
 #include "gailitem.h"
 #include <libgail-util/gailmisc.h>
@@ -131,7 +132,7 @@ gail_item_real_initialize (AtkObject *obj,
   label = get_label_from_container (GTK_WIDGET (data));
   if (GTK_IS_LABEL (label))
     {
-      if (GTK_WIDGET_MAPPED (label))
+      if (gtk_widget_get_mapped (label))
         gail_item_init_textutil (item, label);
       else
         g_signal_connect (label,
@@ -230,13 +231,6 @@ gail_item_get_name (AtkObject *obj)
           if (GTK_IS_MENU (parent))
             {
               attach = gtk_menu_get_attach_widget (GTK_MENU (parent)); 
-
-              if (GTK_IS_OPTION_MENU (attach))
-                {
-                  label = get_label_from_container (attach);
-                  if (GTK_IS_LABEL (label))
-	            return gtk_label_get_text (GTK_LABEL(label));
-                }
               list = gtk_container_get_children (GTK_CONTAINER (parent));
               index = g_list_index (list, widget);
 
